#! /bin/sh

# gnuplot2fig: a shell script to create an XFIG file from a
#              set of gnuplot commands.

# Global variables
GNUPLOT=gnuplot

USAGE() {
	echo "`basename $0` <gnuplot commands file> <xfig output file> [mono]"
	exit 1
}

FIND() {
        test $# -eq 1 || {
		echo "FIND expects a single arg:"
		echo "The executable or shell script to search for."
		exit 1
	}

        type "$1" > /dev/null || {
                echo "Unable to find \"$1\". Please install."
                exit 1
        }
}

RUN_GNUPLOT() {
	test $# -eq 1 || {
		echo "RUN_GNUPLOT expects a single arg:"
		echo "The file containing the gnuplot commands."
		exit 1
	}

	GNUPLOT_EXECUTE=gnuplot.execute.$$
	echo "load \"$1\"; quit" > ${GNUPLOT_EXECUTE}

	${GNUPLOT} ${GNUPLOT_EXECUTE} || echo "gnuplot failed to run!"
	rm -f ${GNUPLOT_EXECUTE}
}

# Set the output target to XFIG and direct output to a file.
BUILD_XFIG_TARGET() {
	test $# -eq 4 || {
		echo "BUILD_XFIG_TARGET expects four args:"
		echo "1. The file containing the gnuplot commands."
		echo "2. The name of the XFIG file."
		echo "3. The (output) file which prepends the commands in 1 "
			"with the XFIG target and XFIG output file."
		echo "4. color/mono output in color?"
		exit 1
	}

	echo "set terminal fig $4 small fontsize 10" > "$3"
	echo "set output \"$2\"" >> "$3"
	cat "$1" >> "$3"
}

# GNUPLOT will output an XFIG file with "normal" text.
# This function ensures that all text is marked as "special".
MANIP_XFIG_FILE() {
	test $# -eq 1 || {
		echo "MANIP_XFIG_FILE expects a single arg:"
		echo "The file containing the xfig dataset."
		exit 1
	}

	TEMPFILE=tmp.$$
	# Lines containing text end "\001".
	# For all such lines:
	# change the 3rd column to -1 (text has default colour rather
	# than the blue, red etc colour of the line).
	# change the 9th column from 0 to 3 (mark as special).
	# A column is defined here as 1 or more chars (not spaces) followed by
	# a space.
	sed '/\001$/{
	s/[^ ]\{1,\} /-1 /3
	s/[^ ]\{1,\} /3 /9
	}' "$1" > ${TEMPFILE}
	mv -f ${TEMPFILE} "$1"
}

# The main logic of the script
COLOR="color"
test $# -eq 3 && {
	test "$3" = "mono" && COLOR="monochrome" || USAGE
} || {
	test $# -eq 2 || USAGE
}

test -r $1 || {
	echo "Unable to read '$1'"
	USAGE
}

FIND ${GNUPLOT}

# Create an XFIG data set as $2
GNUPLOT_COMMANDS=gnuplot.commands.$$
BUILD_XFIG_TARGET "$1" "$2" "${GNUPLOT_COMMANDS}" ${COLOR}

RUN_GNUPLOT "${GNUPLOT_COMMANDS}"
rm -f "${GNUPLOT_COMMANDS}"

# Ensure that all text is marked as "special"
MANIP_XFIG_FILE "$2"

# The end
